/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CAMERA_HPP
#define CAMERA_HPP

#include <v3.hpp>

namespace cpe
{
class p2d;

/** \brief Storage of a camera and screen parameters */
class camera
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  Constructor
        // ********************************************* //
        // ********************************************* //

        /** \brief direct constructor */
        camera(const v3& center=v3(0,0,-2),const v3& direction=v3(0,0,1),const v3& up=v3(0,1,0),const double& dist_screen=2.0,const double& length_screen=4.0);

        // ********************************************* //
        // ********************************************* //
        //  Access parameters
        // ********************************************* //
        // ********************************************* //

        /** \brief get the center of the camera */
        const v3& get_center() const;
        /** \brief get the direction value */
        const v3& get_direction() const;
        /** \brief get the up direction value */
        const v3& get_up() const;
        /** \brief get the dist_screen value */
        const double& get_dist_screen() const;
        /** \brief get the length_of the screen value */
        const double& get_length_screen() const;

        // ********************************************* //
        // ********************************************* //
        //  Compute position on the screen
        // ********************************************* //
        // ********************************************* //

        /** \brief get the screen coordinate given the parametric (u,v) data. (u,v) belongs to [0,1] **/
        v3 get_screen_coordinate(const double& u,const double& v) const;

    private:

        /** \brief internal camera center */
        v3 p;
        /** \brief direction of pointing camera */
        v3 dir;
        /** \brief up direction of the camera */
        v3 up;
        /** \brief internal screen distance to the center */
        double dist_screen;
        /** \brief length of the screen */
        double length_screen;
};
}

#endif
