/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <camera.hpp>
#include <p2d.hpp>

namespace cpe
{
camera::camera(const v3& center,const v3& direction,const v3& up_,const double& dist_screen_,const double& length_screen_)
    :p(center),dir(direction),up(up_),dist_screen(dist_screen_),length_screen(length_screen_){}

const v3& camera::get_center() const
{
    return p;
}
const v3& camera::get_direction() const
{
    return dir;
}
const v3& camera::get_up() const
{
    return up;
}
const double& camera::get_dist_screen() const
{
    return dist_screen;
}
const double& camera::get_length_screen() const
{
    return length_screen;
}
v3 camera::get_screen_coordinate(const double& u,const double& v) const
{
    v3 right=dir.cross(up).normalized();
    v3 x0=p+dir*dist_screen;

    v3 tr_right=right*(u-0.5)*2.0*length_screen;
    v3 tr_up=up*(v-0.5)*2.0*length_screen;

    v3 current=x0+tr_right+tr_up;

    return current;
}

}
