/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <scene3d.hpp>

#include <material.hpp>
#include <camera.hpp>

#include <cstdlib>
#include <sstream>



namespace cpe
{
    scene3d::scene3d(){v_object.clear();v_light.clear();}
    void scene3d::clean_memory()
    {
        for(unsigned int k=0,N=v_object.size();k<N;++k)
        {
            delete v_object[k].first;
            v_object[k].first=0;
        }
        v_object.clear();
    }

    void scene3d::add(const object3d* obj,const material& material)
    {
        v_object.push_back(std::make_pair(obj,material));
    }
    void scene3d::add(const light& light_parameter)
    {
        v_light.push_back(light_parameter);
    }
    unsigned int scene3d::N_object() const
    {
        return v_object.size();
    }
    unsigned int scene3d::N_light() const
    {
        return v_light.size();
    }
    const object3d* scene3d::get_object(const unsigned int k_object) const
    {

        if(k_object>v_object.size())
        {
            //throw error
            std::ostringstream msg;msg<<"Object vector has size "<<v_object.size()<<" and you are trying to access element: "<<k_object;
            throw exception_scene3d(msg.str(),__FILE__,__FUNCTION__,__LINE__);
        }



        return v_object[k_object].first;
    }
    const light& scene3d::get_light(const unsigned int k_light) const
    {

        if(k_light>v_light.size())
        {
            //throw error
            std::ostringstream msg;msg<<"Light vector has size "<<v_light.size()<<" and you are trying to access element: "<<k_light;
            throw exception_scene3d(msg.str(),__FILE__,__FUNCTION__,__LINE__);
        }


        return v_light[k_light];
    }
    const material& scene3d::get_material(const unsigned int k_material) const
    {

        if(k_material>v_object.size())
        {
            //throw error
            std::ostringstream msg;msg<<"Object(/material) vector has size "<<v_object.size()<<" and you are trying to access element: "<<k_material;
            throw exception_scene3d(msg.str(),__FILE__,__FUNCTION__,__LINE__);
        }

        return v_object[k_material].second;
    }

    void scene3d::set_camera(const camera& cam_)
    {
        cam=cam_;
    }
    const camera& scene3d::get_camera() const
    {return cam;}

    void scene3d::set_background_color(const color& background_color_)
    {
        background_color=background_color_;
    }
    const color& scene3d::get_background_color() const
    {return background_color;}

}
