/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#ifndef P2D_HPP
#define P2D_HPP

#include <iostream>
#include <exception_cpe.hpp>

namespace cpe
{
    /** \brief Container for 2D integer positions (pixel position) */
    class p2d
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        p2d();
        /** \brief direct constructor
            \param int x0: x-coordinate
            \param int y0: y-coordinate
        */
        p2d(const int& x0,const int& y0);

        // ********************************************* //
        // ********************************************* //
        //  Coordinate access
        // ********************************************* //
        // ********************************************* //

        /** \brief set (x,y)-coordinate */
        void set(const int& x,const int& y);

        /** \brief get x-coordinate */
        const int& x() const;
        /** \brief get x-coordinate */
        int& x();
        /** \brief get y-coordinate */
        const int& y() const;
        /** \brief get y-coordinate */
        int& y();


        // ********************************************* //
        // ********************************************* //
        //  Operators
        // ********************************************* //
        // ********************************************* //

        /** \brief sum between two positions */
        friend p2d operator+(const p2d& u1,const p2d& u2);
        /** \brief diff between two positions */
        friend p2d operator-(const p2d& u1,const p2d& u2);
        /** \brief internal sum between two positions */
        p2d& operator+=(const p2d& u);
        /** \brief internal diff between two positions */
        p2d& operator-=(const p2d& u);

        // ********************************************* //
        // ********************************************* //
        //  I/O
        // ********************************************* //
        // ********************************************* //

        /** \brief standard output */
        friend std::ostream& operator<<(std::ostream& stream,const p2d& u);


    private:

        // ********************************************* //
        // ********************************************* //
        //  Internal storage
        // ********************************************* //
        // ********************************************* //

        /** \brief internal x-coordinate */
        int internal_x;
        /** \brief internal y-coordinate */
        int internal_y;

    };

    /** \brief Exception class for p2d */
    class exception_p2d : public exception_cpe
    {
    public:
        /** \brief empty constructor */
        exception_p2d():exception_cpe(){}
        /** \brief constructor with error information */
        exception_p2d(const std::string& msg,const std::string& file,const std::string& caller,const int& line):exception_cpe(msg,file,caller,line){}
    };
}


#endif
