/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <color.hpp>


namespace cpe
{

    color::color():internal_r(0),internal_g(0),internal_b(0){}
    color::color(const int& gray):internal_r(gray),internal_g(gray),internal_b(gray){}
    color::color(const int& r,const int& g,const int& b):internal_r(r),internal_g(g),internal_b(b){}


    void color::set(const int& r,const int& g,const int& b){internal_r=r;internal_g=g;internal_b=b;}

    const int& color::r() const{return internal_r;}
    int& color::r(){return internal_r;}
    const int& color::g() const{return internal_g;}
    int& color::g(){return internal_g;}
    const int& color::b() const{return internal_b;}
    int& color::b(){return internal_b;}

    //affichage a l'ecran d'une couleur
    std::ostream& operator<<(std::ostream& stream,const color& c)
    {
        //affiche uniquement "r g b" que l'on concatene dans le flux.
        stream<<c.r()<<" "<<c.g()<<" "<<c.b();
        return stream;
    }

    //interpolation lineaire
    color color::interpolate_linear(const color& c1,const color& c2,const double& alpha)
    {
        //interpolation composante rouge
        double r=(1-alpha)*c1.r()+alpha*c2.r();
        //interpolation composante verte
        double g=(1-alpha)*c1.g()+alpha*c2.g();
        //interpolation composante bleue
        double b=(1-alpha)*c1.b()+alpha*c2.b();


        //tronque valeurs entre [0,255]
        if(r<0)r=0;if(r>255) r=255;
        if(g<0)g=0;if(g>255) g=255;
        if(b<0)b=0;if(b>255) b=255;

        //rappel: static_cast<int>(r) revient ici au même que int(r). Il s'agit d'un cast d'un "double" vers un "int".
        return color(static_cast<int>(r),
                     static_cast<int>(g),
                     static_cast<int>(b));
    }

    //tronque chaque composante de couleur entre 0 et 255
    color color::clamped() const
    {
        return color(std::min(std::max(internal_r,0),255),
                     std::min(std::max(internal_g,0),255),
                     std::min(std::max(internal_b,0),255));
    }

    color color::interpolate_linear(const std::vector<color>& v_color,const std::vector<double>& weights)
    {
        // size verification
        if(v_color.size()!=weights.size())
            throw exception_color("v_color and weight have different size ",__FILE__,__FUNCTION__,__LINE__);


        //convert to double to avoid unsigned char truncation
        std::vector<double> v_color_double;
        for(unsigned int k=0,N=v_color.size();k<N;++k)
        {
            v_color_double.push_back(static_cast<double>(v_color[k].r()));
            v_color_double.push_back(static_cast<double>(v_color[k].g()));
            v_color_double.push_back(static_cast<double>(v_color[k].b()));
        }



        for(unsigned int k=0,N=v_color.size();k<N;++k)
            for(unsigned int k_dim=0;k_dim<3;++k_dim)
                v_color_double[3*k+k_dim]*=weights[k];

        double final[3]={0.0,0.0,0.0};
        for(unsigned int k=0,N=v_color.size();k<N;++k)
            for(unsigned int k_dim=0;k_dim<3;++k_dim)
                final[k_dim] += v_color_double[3*k+k_dim];

        for(unsigned int k_dim=0;k_dim<3;++k_dim)
        {
            if(final[k_dim]<0)
                final[k_dim]=0;
            if(final[k_dim]>255)
                final[k_dim]=255;
        }

        return color(final[0],final[1],final[2]);

    }


    color operator+(const color& c0,const color& c1)
    {
        color c=c0;
        c+=c1;
        return c;
    }

    color& color::operator+=(const color& c)
    {
        internal_r+=c.r();
        internal_g+=c.g();
        internal_b+=c.b();

        return *this;
    }

    color operator*(const double& alpha,const color& c)
    {
        color c2=c;c2*=alpha;return c2;
    }

    color operator*(const color& c,const double& alpha)
    {
        return alpha*c;
    }

    color& color::operator*=(const double& alpha)
    {
        internal_r=alpha*internal_r;
        internal_g=alpha*internal_g;
        internal_b=alpha*internal_b;

        return *this;
    }

}
