/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef PLANE_HPP
#define PLANE_HPP

#include <object3d.hpp>

namespace cpe
{

    /** \brief Object3d of type (infinite) Plane (position+normal). Analytically compute intersection with a ray */
    class plane : public object3d
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        plane();
        /** \brief direct constructor (position+normal) */
        plane(const v3& x0,const v3& n);

        /** \brief destructor */
        virtual ~plane();

        // ********************************************* //
        // ********************************************* //
        //  Accessor
        // ********************************************* //
        // ********************************************* //

        /** \brief get the normal of the plane */
        const v3& normal() const;

        /** \brief get the position of the plane */
        const v3& position() const;


        // ********************************************* //
        // ********************************************* //
        //  INTERSECTION
        // ********************************************* //
        // ********************************************* //

        /** \brief Analytic intersection between a ray and the plane */
        virtual std::vector<intersection_data> intersect(const ray& seg) const;

    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL DATA
        // ********************************************* //
        // ********************************************* //


        /** \brief internal position of the plane (not unique for a given plane) */
        const v3 x0;
        /** \brief internal normal of the plane */
        const v3 n;
    };
}

#endif
