/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#ifndef SPHERE_HPP
#define SPHERE_HPP

#include <object3d.hpp>
#include <v3.hpp>

namespace cpe
{

    class ray;

    /** \brief Object3d of type Sphere (center+radius). Analytically compute intersection with a ray */
    class sphere : public object3d
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        sphere();
        /** \brief direct constructor (center+radius) */
        sphere(const v3& x0,const double& r);

        /** \brief desctructor */
        virtual ~sphere();


        // ********************************************* //
        // ********************************************* //
        //  Accessor
        // ********************************************* //
        // ********************************************* //

        /** \brief get radius */
        const double& radius() const;

        /** \brief get center */
        const v3& center() const;

        // ********************************************* //
        // ********************************************* //
        //  INTERSECTION
        // ********************************************* //
        // ********************************************* //

        /** \brief Analytic intersection between a ray and the sphere */
        virtual std::vector<intersection_data> intersect(const ray& seg) const;

    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL DATA
        // ********************************************* //
        // ********************************************* //

        /** \brief internal center of the sphere */
        v3 x0;
        /** \brief internal radius of the sphere */
        double r;
    };
}

#endif
