/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <plane.hpp>
#include <ray.hpp>

#include <cmath>

#include <intersection_data.hpp>

namespace cpe
{

    plane::plane():x0(0.0,0.0,0.0),n(0.0,0.0,1.0){}
    plane::plane(const v3& _x0,const v3& _n):x0(_x0),n(_n){}
    plane::~plane(){}


    const v3& plane::normal() const {return n;}
    const v3& plane::position() const {return x0;}

    std::vector<intersection_data> plane::intersect(const ray& seg) const
    {
        //vecteur d'intersection (il n'y aura au plus qu'une intersectionà
        std::vector<intersection_data> inter;

        double epsilon=1e-8;

        //proj=<u.n>
        double proj=seg.u().dot(n);

        //cas de rayons parallele au plan
        if(std::fabs(proj)<epsilon)
            return inter;

        //valeur du parametre locale
        double t=(x0-seg.x0()).dot(n)/proj;

        //ajout de cette intersection
        inter.push_back(intersection_data( seg(t), n ,t ));

        return inter;
    }

}
