/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/





#ifndef RAY_HPP
#define RAY_HPP

#include <v3.hpp>

namespace cpe
{

class camera;

/** \brief Straight ray (initial position + direction) used for ray-tracing algorithm */
class ray
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        ray();
        /** \brief direct constructor (position+direction) */
        ray(const v3& _x0,const v3& _u);

        // ********************************************* //
        // ********************************************* //
        //  DATA ACCESS
        // ********************************************* //
        // ********************************************* //

        /** \brief get initial position */
        const v3& x0() const;
        /** \brief get initial position */
        v3& x0();
        /** \brief get direction */
        const v3& u() const;
        /** \brief get direction */
        v3& u();

        // ********************************************* //
        // ********************************************* //
        //  RAY EVALUATION
        // ********************************************* //
        // ********************************************* //

        /** \brief evaluate 3D position at x0+t*u */
        v3 operator()(const double& t) const;

        // ********************************************* //
        // ********************************************* //
        //  HELPER RAY CONSTRUCTION
        // ********************************************* //
        // ********************************************* //

        /** \brief helper static method to generate a ray given a camera position and a screen definition */
        static ray generate_ray_from_camera(const camera& cam,const double& alpha_x,const double& alpha_y);

        /** \brief Compute the reflected ray of the current one given the normal */
        ray reflected_ray(const v3& n) const;

        /** \brief offset the ray by epsilon in the direction of the normal */
        void offset(const double& epsilon=1e-8);


    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL DATA
        // ********************************************* //
        // ********************************************* //

        /** \brief internal initial position of the ray */
        v3 internal_x0;
        /** \brief internal direction of the ray */
        v3 internal_u;

};
}

#endif
