/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef RAY_TRACER_HPP
#define RAY_TRACER_HPP

#include <vector>


namespace cpe
{

    struct intersection_object;
    struct render_parameters;
    class image;
    class color;
    class ray;
    class scene3d;
    class v3;


    /** \brief Helper class for running ray-tracing algorithm */
    class ray_tracer
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  RAY-TRACING
        // ********************************************* //
        // ********************************************* //

        /** \brief run ray-tracing algorithm given a 3D scene and a picture */
        static void trace(const scene3d& scene,const render_parameters& render_param,image* pic);


    //private:

        // ********************************************* //
        // ********************************************* //
        //  HELPER-METHOD
        // ********************************************* //
        // ********************************************* //

        /** \brief throw a ray in the scene along the given ray
            \return the color associated to this ray
        */
        static color throw_ray(const scene3d& scene,const ray& ray_current,const render_parameters& render_param);

        /** \brief find the intersection between a given ray and the first object in the 3D scene */
        static intersection_object find_inter(const ray& ray_current,const scene3d& scene);

        /** \brief throw ray to the direction of every lights
            \return the vector containing the distances from current_position to the lights (distance=-1 if we are in the shadow of an other object)
        */
        static std::vector<double> ray_to_light(const v3& current_position,const scene3d& scene);
        /** \brief compute the color associated to the vector of distance from every light sources */
        static color compute_color(const std::vector<double>& t_light,const intersection_object& inter,const scene3d& scene,const render_parameters& render_param);

        /** \brief compute the color associated to a intersection */
        static color find_intersection_color(const intersection_object& inter,const scene3d& scene,const render_parameters& render_param);


    };

}

#endif
