/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <ray.hpp>
#include <camera.hpp>

namespace cpe
{
    ray::ray(){}
    ray::ray(const v3& _x0,const v3& _u):internal_x0(_x0),internal_u(_u){}

    const v3& ray::x0() const{return internal_x0;}
    v3& ray::x0(){return internal_x0;}
    const v3& ray::u() const{return internal_u;}
    v3& ray::u(){return internal_u;}

    v3 ray::operator()(const double& t) const
    {
        return internal_x0+t*internal_u;
    }

    ray ray::reflected_ray(const v3& n) const
    {

        v3 reflection=internal_u-2*internal_u.dot(n)*n;
        return ray(internal_x0,reflection.normalized());
    }

    void ray::offset(const double& epsilon)
    {
        internal_x0 += epsilon*internal_u;
    }

    ray ray::generate_ray_from_camera(const camera& cam,const double& alpha_x,const double& alpha_y)
    {
        return ray(cam.get_center(),(cam.get_screen_coordinate(alpha_x,alpha_y)-cam.get_center()).normalized());
    }
}
