/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <ray_tracer.hpp>

#include <image.hpp>
#include <scene3d.hpp>
#include <color.hpp>
#include <ray.hpp>
#include <p2d.hpp>
#include <intersection_object.hpp>
#include <material.hpp>
#include <shading.hpp>
#include <camera.hpp>
#include <vertex_shader.hpp>
#include <vertex.hpp>
#include <render_parameters.hpp>
#include <cmath>


namespace cpe
{


    void ray_tracer::trace(const scene3d& scene,const render_parameters& render_param,image *pic)
    {

        const int& Nx=pic->get_Nx();
        const int& Ny=pic->get_Ny();


        // ********************************************* //
        // ********************************************* //
        //  TO DO: Réalisez l'antialiasing
        //
        //
        // ********************************************* //
        // ********************************************* //


        for(int ky=0;ky<Ny;++ky)
        {
            for(int kx=0;kx<Nx;++kx)
            {

                double ux=static_cast<double>(kx)/(Nx-1.0);
                double uy=static_cast<double>(ky)/(Ny-1.0);

                ray ray_current=ray::generate_ray_from_camera(scene.get_camera(),ux,uy);
                color col=throw_ray(scene,ray_current,render_param);

                pic->set(p2d(kx,ky),col);

            }
        }

    }


    intersection_object ray_tracer::find_inter(const ray& ray_current,const scene3d& scene)
    {
        double epsilon=1e-8;
        intersection_object first_intersection;


        // ********************************************* //
        // ********************************************* //
        //  TO DO: Calcul de l'illumination à une intersection donnée:
        //
        //
        // Algorithm:
        //
        // |
        // | Initialise intersection_object: inter;
        // |
        // | Pour tout objet Obj de la scene
        // |
        // |   vec_inter = Obj.intersection(ray_current)
        // |
        // |   Pour tout k de vec_inter
        // |     Si vec_inter[k] est située avant toutes les autres
        // |       inter=vec_inter[k]
        // |   Fin Pour
        // | Fin Pour
        // |
        // | retourner inter
        // |
        //
        // ********************************************* //
        // ********************************************* //


        for(unsigned int k=0,N=scene.N_object();k<N;++k)
        {
            const object3d* obj=scene.get_object(k);
            std::vector<intersection_data> inter=obj->intersect(ray_current);
            if(inter.size()>0)
            {
                return intersection_object(inter[0],k);
            }
        }
    }

    color ray_tracer::find_intersection_color(const intersection_object& inter,const scene3d& scene,const render_parameters& render_param)
    {
        //get intersection for every light
        std::vector<double> t_light=ray_to_light(inter.inter_data.x,scene);

        //compute the color relative to the lights
        color new_color=compute_color(t_light,inter,scene,render_param);

        return new_color;
    }

    std::vector<double> ray_tracer::ray_to_light(const v3& current_position,const scene3d& scene)
    {
        std::vector<double> distance_to_light(scene.N_light(),-1.0);

        // ********************************************* //
        // ********************************************* //
        //  TO DO: Calcul de la distance d'un point à
        // toutes les sources de lumières.
        //
        //
        // Algorithm:
        //
        // |
        // |
        // | Pour toutes lumieres k
        // |
        // |   lance rayon (current_position,light[k])
        // |   Si rayon intersecte un objet avant la lumière
        // |       vec_inter[k]=-1;
        // |   Sinon
        // |       vec_inter[k]=distance(current_position,light[k])
        // | Fin Pour
        // |
        // | retourner vec_inter
        // |
        //
        // ********************************************* //
        // ********************************************* //


        return distance_to_light;
    }



    color ray_tracer::compute_color(const std::vector<double>& t_light,const intersection_object& inter,const scene3d& scene,const render_parameters& render_param)
    {
        // ********************************************* //
        // ********************************************* //
        //  TO DO: Calcul de la couleur associée à une intersection
        //
        //
        // Algorithm:
        //
        // |Si pas d'intersection: return color_background
        // |
        // | Pour toutes les sources de lumière k
        // |   vertex v=vertex(inter.position,
        // |                   inter.normale,
        // |                   couleur objet courant);
        // |
        // |   Si t_light[k]>0
        // |      shading shad=shading de l'objet courant
        // |   Sinon
        // |      shading shad=composante ambiante uniquement de l'objet courant
        // |
        // |   color=shading(vertex,shad,light[k],camera)
        // |
        // | Fin pour
        // |
        // | retourner Somme des couleurs de toute les lampes
        // |
        //
        // ********************************************* //
        // ********************************************* //



        return scene.get_material(inter.id_object).c;
    }

    color ray_tracer::throw_ray(const scene3d& scene,const ray& ray_current,const render_parameters& render_param)
    {

        //background color;
        color col=scene.get_background_color();

        //intersection ray from camera
        intersection_object inter=find_inter(ray_current,scene);
        if(inter.id_object>=0)
        {
            col=find_intersection_color(inter,scene,render_param);


            // ********************************************* //
            // ********************************************* //
            //  TO DO: Lancé de rayons reflechies dans le cas de la reflection
            //
            // Algorithm:
            //
            // Pour k=1:max_reflection
            //   ray_current=(inter,reflection(ray_current))
            //   Si intersection avec objets
            //     couleur += alpha^k * couleur_de_l_intersection
            // Fin pour
            //
            // ********************************************* //
            // ********************************************* //

        }

        return col;

    }



}
