
/** TP 4ETI - CPE Lyon - 2012 */

#include <iostream>
#include <cstdlib>

#include <image.hpp>
#include <color.hpp>
#include <p2d.hpp>

#include <scene3d.hpp>
#include <ray.hpp>
#include <object3d.hpp>
#include <ray_tracer.hpp>
#include <sphere.hpp>
#include <plane.hpp>
#include <material.hpp>
#include <shading.hpp>
#include <camera.hpp>
#include <light.hpp>
#include <render_parameters.hpp>


using namespace cpe;



int main(int argc,char *argv[])
{

    std::cout<<"****"<<std::endl;
    std::cout<<"run "<<argv[0]<<" with "<<argc-1<<" parameters ... \n"<<std::endl;

    try
    {
        //common shading parameters
        shading shad(0.2,1.0,0.5,64.0);

        //render parameters
        render_parameters render_param(3,5,true);

        //define an empty scene
        scene3d scene;


        //camera
        scene.set_camera(camera(v3(0,0,-2),v3(0,0,1),v3(0,1,0),2.0,1.2));
        //add some spheres
        scene.add(new sphere(v3(-0.5,1,2),0.9),material(color(150,150,255),shad));
        scene.add(new sphere(v3(1,1-0.2,1),0.2),material(color(255,0,0),shad));
        scene.add(new sphere(v3(+1.7,-0.2,5),0.3),material(color(100,255,0),shad));
        //add a plane
        scene.add(new plane(v3(0,+1,0),v3(0,-1,0)),material(color(255,255,255),shad));

        //add a light
        scene.add(light(v3(6,-5,0)));

        //background color
        scene.set_background_color(color(0,0,0));


        //generate an empty picture
        image pic(200);

        //run ray-tracing
        ray_tracer::trace(scene,render_param,&pic);

        //delete allocated memory
        scene.clean_memory();

        //export picture in ppm format
        pic.export_file("my_pic.ppm");

    }
    catch(exception_cpe e)
    {
        std::cout<<"**********************************"<<std::endl;
        std::cout<<"**********************************"<<std::endl;
        std::cout<<"Exception found"<<std::endl;
        std::cout<<e.info()<<std::endl;
        std::cout<<"**********************************"<<std::endl;
        std::cout<<"**********************************"<<std::endl;
        exit(1);
    }


    std::cout<<"Exit Main"<<std::endl;

    return 0;
}
